#R script for:
#Title: Consistent population declines but idiosyncratic range shifts in Alpine orchids under global change
#Authors: Costanza Geppert, Giorgio Perazza, Robert J. Wilson, Alessio Bertolli, Filippo Prosser, Giuseppe Melchiori, Lorenzo Marini
#Journal: Nature Communications 
#Year: 2020
#load required packages####
library(nlme)
library(lme4)
library(car)
library(DHARMa)
library(lsmeans)
library(multcompView)
library(WRS2)
library(plyr)

#1 Local survival based on resurveys ####
resurvey<-read.table("C:data\\Resurvey.txt", header = T) # import data
resurvey$Habitat_alteration<-as.factor(resurvey$Habitat_alteration) # set Habitat alteration (0,1) as factor
resurvey1<-ddply(resurvey, "Species", transform, Elevation.std = scale(Elevation)) # standardize elevation to mean 0 and SD = 1 within species to make the elevational distribution comparable among species
resurvey_sub<-subset(resurvey1, Delta_jd<30 & Count_hist>4 & Delta_year>4) # subset excluding observations with more than 30 day differences, species recorded less than 5 times in the initial survey, and sites revisited after less than 5 years (see Methods and reporting summary for exclusion criteria)


mod1<-glmer(Survival~scale(Delta_year)+scale(log_Abu)+Habitat_preference+Elevation.std+Habitat_alteration+
              (1|Species),data=resurvey_sub, family=binomial) # run model assuming binomial distribution and species as random factor

simulationOutput <- simulateResiduals(fittedModel = mod1)
plotSimulatedResiduals(simulationOutput = simulationOutput, quantreg = T) # visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
vif(mod1) # assess collinearity
summary(mod1)
Anova(mod1, type = "III")


#2 Temporal trends in population size###########
Abundance<-read.table("C:data\\Abundance.txt",header=TRUE) #import data
Abundance$Period<-as.factor(Abundance$Period) # set period (0 = 1990-2003; 1 = 2004-2017) as factor
# Separate models for each habitat preference category
# Models assume Poisson distribution with species and an observation-level random factor as crossed random effects (see Methods)
forest<-subset(Abundance, Forest==1) # 1) select orchid species belonging to the forest habitat preference group
modfor<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=forest)

residuals_dharma <- simulateResiduals(modfor)
plot(residuals_dharma) # visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modfor)
Anova(modfor)

generalist<-subset(Abundance, Generalist==1) # 2) select orchid species belonging to the generalist group
modgen<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=generalist)

residuals_dharma <- simulateResiduals(modgen)
plot(residuals_dharma)# visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modgen)
Anova(modgen)

grassland<-subset(Abundance, Grassland==1) # 3) select orchid species belonging to the grassland habitat preference group
modgra<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=grassland)

residuals_dharma <- simulateResiduals(modgra)
plot(residuals_dharma)# visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modgra)
Anova(modgra)

semi<-subset(Abundance, Semi_natural_grassland==1) #4) select orchid species belonging to the semi-natural grassland habitat preference group
modsem<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=semi)

residuals_dharma <- simulateResiduals(modsem)
plot(residuals_dharma) # visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modsem)
Anova(modsem)

alpine<-subset(Abundance, Subalpine_Alpine==1) # 5) select orchid species belonging to the alpine habitat preference group 
modalp<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=alpine)

residuals_dharma <- simulateResiduals(modalp)
plot(residuals_dharma) # visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modalp)
Anova(modalp)

wetland<-subset(Abundance, Wetland==1) #6) select orchid species belonging to the wetland habitat preference group 
modwet<-glmer(Abundance~
                scale(I(Elevation.std^2))+scale(Elevation.std)*scale(Year)+
                (1|Species)+(1|ID),family=poisson,
              	data=wetland)

residuals_dharma <- simulateResiduals(modwet)
plot(residuals_dharma) # visualize residuals 1) qq-plot to detect overall deviations from the expected distribution; 2) residuals against the predicted value
summary(modwet)
Anova(modwet)
 
# 3 Range shifts ####
# Density function extracting: 1) rear and 2) leading edge (the 5% and 95% quantiles of the density distribution), and 3) optimum (the highest peak of the density distribution).
peak<- function(elevation){
  d<-density(elevation, na.rm=TRUE)
  optimum<-as.vector(d$x[d$y==max(d$y)]) ###single highest peak optimum
  library(spatstat)
  edge<-as.vector(quantile(d, probs = c(0.05,  0.95), names = TRUE)) ##quantile density
  a<-c( edge, optimum)
  print(a)
}

Abundance_hist<-subset(Abundance, Period==0) # select records from 1990 to 2003 (historical period)
Abundance_rec<-subset(Abundance, Period==1) # select records from 2004 to 2017 (current period)
tapply(Abundance_hist$Elevation, Abundance_hist$Species, peak) # extracting rear, leading and optimum measures in the historical period
tapply(Abundance_rec$Elevation, Abundance_rec$Species, peak) # extracting rear, leading and optimum measures in the current period

# 1) using the function "peak", it is now possible to calculate the annual shift rates at rear, leading and optimum position per species
# NB annual shift rate = (current measures of elevational range - historical measures of elevational range) / 14 
# 2) import the so obtained shift rates, name them: Rear_shift; Leading_shift and Optimum_shift
# 3) run models on obtained shifts at rear, leading and optimum position as:

mod2<-lm(Rear_shift~Habitat_preference, data=shift)
summary(mod2)
qqPlot(residuals(mod2)) # visualize quantile-quantile plot
lsmeans(mod2, pairwise~Habitat_preference, adjust="tukey") # Tukey-adjusted comparisons 
mod3<-lm(Leading_shift~Habitat_preference, data=shift)
qqPlot(residuals(mod3)) # visualize quantile-quantile plot
summary(mod3)
lsmeans(mod3, pairwise~Habitat_preference, adjust="tukey") # Tukey-adjusted comparisons 
mod4<-lm(Optimum_shift~Habitat_preference, data=shift)
qqPlot(residuals(mod4)) # visualize quantile-quantile plot
summary(mod4)
lsmeans(mod4, pairwise~Habitat_preference, adjust="tukey") # Tukey-adjusted comparisons 

#Deciles per species
Abundance<-read.table("C:data\\Abundance.txt",header=TRUE) #import data
Abundance$Elevation_rate<-Abundance$Elevation/14 # divide by 14 years to obtain annual rate
Abundance$Period2<-Abundance$Period*(-1)
Abundance$Period2<-as.factor(Abundance$Period2)

# example with one species:
# 1) A. pyramidalis
ap<-subset(Abundance,Species=="Anacamptis_pyramidalis") # select only records of A. pyramidalis
a<-qcomhd(Elevation_rate~Period2,data=ap,q = seq(.1, .9, by=.1)) # compare deciles via the Harrell-Davis estimator
b<-a$partable
colnames(b)[6]  <- "difference"
#visualize the shift for each decile between historical and current elevation distribution
g1<-ggplot(b, aes(x=q, y=difference)) + 
  geom_pointrange(aes(ymin=ci.low, ymax=ci.up ), size=0.1) +
  ggtitle("Ana pyr")+
  geom_point()+
  geom_abline(intercept = 0, slope = 0, size=0.2, linetype="dotted")+
  theme_classic()+
  theme(axis.title.x = element_blank(),
        axis.title.y = element_blank())

g1<-g1+theme(axis.text=element_text(size=8))+ 
  theme(plot.title=element_text(size=8)) +
  ylim(1.05*(-max(abs(b$difference)+10)), 1.05*max(abs(b$difference)+5))

g1<-g1+theme(plot.margin = margin(0.0, 0.1, 0, 0, "cm"))+
  theme(plot.title = element_text(vjust = - 3, hjust=0.5))+
  theme(axis.text.x = element_blank(),
        axis.line = element_line(colour = 'black', size = 0.2))
g1

ap1<-subset(ap,Period==1) # select records of A. pyramidalis in the Current period
el1<-ap1$Elevation 
el1<-as.vector(el1) 
ap0<-subset(ap,Period==0) # select records of A. pyramidalis in the Historical period
el0<-ap0$Elevation
el0<-as.vector(el0)
ks.test(el0, el1) # perform Kolmogorov-Smirnov test comparing the elevation in the current vs. the elevation in the historical period
# NB obtained p-values need to be corrected using Benjamini Hochberg correction with the R package "sgof"


